# This file was auto-generated by Fern from our API Definition.

from . import (
    account_base,
    accounts,
    audit,
    auth,
    auth_base,
    bridges,
    capabilities,
    capabilities_base,
    common,
    credentials,
    integration_base,
    integration_points,
    integrations,
    management,
    mappings,
    member_base,
    members,
    meta,
    operation_base,
    operation_id_generated,
    operations,
    organization,
    organization_base,
    organization_webhook_base,
    organization_webhook_events,
    organization_webhooks,
    permissions,
    permissionset,
    permissionset_base,
    providers_generated,
    role_base,
    roles,
    status,
    sub_orgs,
    token_base,
    tokens,
    usage,
)
from .account_base import AccountId
from .accounts import (
    Account,
    CreateAccountRequest,
    CreateAccountResponse,
    CreateAccountResponseResult,
    GetAccountResponse,
    ListAccountsResponse,
    PatchAccountResponse,
    UpdateAccountRequest,
    UpdateAccountResponse,
)
from .audit import Audit, AuditType, HttpMethod, ListAuditEventsResponse
from .auth import ChangePasswordRequest
from .auth_base import (
    AuthCode,
    ChangePasswordResponse,
    ChangePasswordResponseResult,
    LogonRequest,
    LogonResponse,
    LogonResponseResult,
)
from .bridges import (
    BridgeGroup,
    BridgeGroupId,
    BridgeLocalConfig,
    BridgeLocalStats,
    BridgeStatus,
    CreateBridgeRequest,
    CreateBridgeResponse,
    CreateBridgeResponseResult,
    GetBridgeResponse,
    GetBridgeStatusResponse,
    ListBridgesResponse,
    PatchBridgeResponse,
    UpdateBridgeRequest,
    UpdateBridgeResponse,
)
from .capabilities import (
    Connector,
    ConnectorOrId,
    FilterOperation,
    FilterType,
    GetProviderCapabilitiesExpandOptions,
    ListConnectorCapabilitiesExpandOptions,
    ListConnectorsCapabilitiesResponse,
    ListProviderCapabilitiesExpandOptions,
    ListProvidersCapabilitiesResponse,
    ProviderAvailability,
    ProviderCapabilities,
    ProviderCapabilitiesResponse,
    ProviderEnvironment,
    ProviderFilter,
    ProviderOperations,
    ProviderRelease,
    RequestBody,
)
from .capabilities_base import CategoryId, ProviderId
from .common import (
    ActionId,
    BadGatewayError,
    BadRequestError,
    Base,
    ConflictError,
    ForbiddenError,
    GatewayTimeoutError,
    Id,
    InternalServerError,
    MethodNotAllowedError,
    NotFoundError,
    NotImplementedError,
    ParameterLocation,
    PatchOp,
    PatchOperation,
    Problem,
    ProblemCause,
    ProblemContext,
    ProblemDetails,
    ProblemRelation,
    ProblemType,
    ProblematicParameter,
    ResourceId,
    ResourceReference,
    ResourceType,
    ServiceUnavailableError,
    TooManyRequestsError,
    UnauthorizedError,
    UnsupportedMediaTypeError,
)
from .credentials import (
    AwsCredential,
    AwsCredentialId,
    BasicCredential,
    BasicCredentialId,
    CreateCredentialRequest,
    CreateCredentialResponse,
    Credential,
    CredentialBase,
    CredentialConfig,
    CredentialConfigNoSecret,
    CredentialConfig_Aws,
    CredentialConfig_Basic,
    CredentialConfig_OAuthClient,
    CredentialConfig_Secret,
    CredentialConfig_Token,
    CredentialId,
    CredentialResponse,
    CredentialType,
    GetCredentialResponse,
    ListCredentialsResponse,
    LookupCredentialResponse,
    ManagedType,
    OAuthClientCredential,
    OAuthClientCredentialId,
    OwnerType,
    PatchCredentialResponse,
    SecretCredential,
    SecretCredentialId,
    TokenCredential,
    TokenCredentialId,
    UpdateCredentialRequest,
    UpdateCredentialResponse,
)
from .integration_base import IntegrationId
from .integration_points import (
    AdditionalMappingTemplate,
    AdditionalMappingType,
    CreateIntegrationPointRequest,
    CreateIntegrationPointResponse,
    GetIntegrationPointResponse,
    IntegrationEnvironments,
    IntegrationPoint,
    IntegrationPointId,
    ListIntegrationPointsResponse,
    MappingChainTemplate,
    PatchIntegrationPointResponse,
    UpdateIntegrationPointRequest,
    UpdateIntegrationPointResponse,
)
from .integrations import (
    BridgeSelector,
    BridgeSelector_Id,
    BridgeSelector_Labels,
    CreateIntegrationRequest,
    CreateIntegrationResponse,
    CreateIntegrationResponseResult,
    GetIntegrationResponse,
    Integration,
    ListAccountIntegrationsResponse,
    ListIntegrationOptions,
    ListIntegrationsResponse,
    PatchIntegrationResponse,
    UpdateIntegrationRequest,
    UpdateIntegrationResponse,
    VerifyIntegrationRequest,
    WebhookConfig,
    WebhookEvent,
    WebhookItem,
)
from .management import AdditionalMapping, MappingChain, MappingDataType
from .mappings import (
    ApplyMappingRequest,
    ApplyMappingResponse,
    ApplyMappingResponseResult,
    CreateMappingRequest,
    CreateMappingResponse,
    CreateMappingResponseResult,
    GetMappingResponse,
    ListMappingsResponse,
    Mapping,
    MappingBase,
    MappingId,
    PatchMappingResponse,
    UpdateMappingRequest,
    UpdateMappingResponse,
)
from .member_base import (
    CreateMemberRequest,
    CreateMemberResponse,
    CreateMemberResponseResult,
    Member,
    MemberId,
    MemberOptions,
    Options,
    State,
)
from .members import (
    GetMemberResponse,
    ListMembersResponse,
    PatchMemberResponse,
    UpdateMemberRequest,
    UpdateMemberResponse,
)
from .meta import GetOpenApiSpecResponse
from .operation_base import (
    AsyncOperationRequestId,
    Operation,
    OperationError,
    OperationInput,
    OperationSchedule,
    OperationStatus,
)
from .operation_id_generated import OperationId
from .operations import ListOperationsResponse
from .organization import PatchOrganizationResponse, UpdateOrganizationRequest, UpdateOrganizationResponse
from .organization_base import (
    CreateOrganizationRequest,
    CreateOrganizationResponse,
    CreateOrganizationResponseResult,
    Environment,
    GetOrganizationResponse,
    ListOrganizationResponse,
    Organization,
    OrganizationId,
    OrganizationOptions,
    OrganizationType,
)
from .organization_webhook_base import WebhookFilter, WebhookId
from .organization_webhook_events import OrganizationWebhookPayload
from .organization_webhooks import (
    CreateOrganizationWebhookRequest,
    CreateOrganizationWebhookResponse,
    GetOrganizationWebhookResponse,
    ListOrganizationWebhooksResponse,
    OrganizationWebhook,
    OrganizationWebhookSecret,
    PatchOrganizationWebhookResponse,
    UpdateOrganizationWebhookRequest,
    UpdateOrganizationWebhookResponse,
)
from .permissions import Permission
from .permissionset import (
    AccountsActions,
    AccountsPermissions,
    AlarmPoliciesActions,
    AlarmPoliciesPermissions,
    AlarmsActions,
    AlarmsPermissions,
    ApiPermissionMap,
    AuditActions,
    AuditPermissions,
    AuthActions,
    AuthPermissions,
    BillingActions,
    BillingPermissions,
    BridgesActions,
    BridgesPermissions,
    CredentialsActions,
    CredentialsPermissions,
    GetPermissionSetResponse,
    IntegrationPointsActions,
    IntegrationPointsPermissions,
    IntegrationsActions,
    IntegrationsPermissions,
    ListPermissionSetsResponse,
    MappingsActions,
    MappingsPermissions,
    MembersActions,
    MembersPermissions,
    OperationsActions,
    OperationsPermissions,
    OrganizationActions,
    OrganizationPermissions,
    PermissionSet,
    PermissionSetActions,
    PermissionSetPermissions,
    ReadWriteActions,
    ReadWritePermissions,
    ResourceRestrictions,
    RolesActions,
    RolesPermissions,
    StatusActions,
    StatusPermissions,
    SubOrgsActions,
    SubOrgsPermissions,
    TokensActions,
    TokensPermissions,
    TransformsActions,
    TransformsPermissions,
    WebhooksActions,
    WebhooksPermissions,
)
from .permissionset_base import Permissions
from .providers_generated import (
    ArmisCredential,
    ArmisCredential_Token,
    ArmisCredential_TokenId,
    AssetsArmisCentrix,
    AssetsArmisCentrixMock,
    AssetsArmisDataset,
    AssetsAxonius,
    AssetsCrowdStrike,
    AssetsNozomiVantage,
    AssetsNozomiVantageDataset,
    AssetsNozomiVantageMock,
    AssetsQualysCloud,
    AssetsQualysCloudDataset,
    AssetsQualysCloudMock,
    AssetsServiceNow,
    AssetsServiceNowDataset,
    AssetsServiceNowMock,
    AssetsSevco,
    AssetsTaniumCloud,
    AssetsTaniumCloudDataset,
    AssetsTaniumCloudMock,
    AutotaskApiIntegrationCodeCredential,
    AutotaskApiIntegrationCodeCredential_Secret,
    AutotaskApiIntegrationCodeCredential_SecretId,
    AutotaskSecretCredential,
    AutotaskSecretCredential_Secret,
    AutotaskSecretCredential_SecretId,
    AwsS3Credential,
    AwsS3Credential_Aws,
    AwsS3Credential_AwsId,
    AwsSecurityLakeCredential,
    AwsSecurityLakeCredential_Aws,
    AwsSecurityLakeCredential_AwsId,
    AwsSqsCredential,
    AwsSqsCredential_Aws,
    AwsSqsCredential_AwsId,
    AxoniusCredential,
    AxoniusCredential_Basic,
    AxoniusCredential_BasicId,
    AzureBlobCredential,
    AzureBlobCredential_Token,
    AzureBlobCredential_TokenId,
    AzureMonitorLogsCredential,
    AzureMonitorLogsCredential_Token,
    AzureMonitorLogsCredential_TokenId,
    CloudSecurityCrowdStrike,
    CloudSecurityDefender,
    CrowdStrikeCredential,
    CrowdStrikeCredential_OAuthClient,
    CrowdStrikeCredential_OAuthClientId,
    CrowdstrikeHecCredential,
    CrowdstrikeHecCredential_Token,
    CrowdstrikeHecCredential_TokenId,
    CustomFieldMapping,
    DefenderCredential,
    DefenderCredential_OAuthClient,
    DefenderCredential_OAuthClientId,
    EdrCrowdStrike,
    EdrDefender,
    EdrMalwarebytes,
    EdrSentinelOne,
    EdrSophos,
    EdrTanium,
    ElasticsearchAuthOptions,
    ElasticsearchCredential,
    ElasticsearchCredential_Basic,
    ElasticsearchCredential_BasicId,
    ElasticsearchCredential_OAuthClient,
    ElasticsearchCredential_OAuthClientId,
    ElasticsearchCredential_Token,
    ElasticsearchCredential_TokenId,
    ElasticsearchSharedSecret,
    ElasticsearchSharedSecret_Secret,
    ElasticsearchSharedSecret_SecretId,
    EntraIdCredential,
    EntraIdCredential_OAuthClient,
    EntraIdCredential_OAuthClientId,
    FreshdeskCredential,
    FreshdeskCredential_Token,
    FreshdeskCredential_TokenId,
    GcsCredential,
    GcsCredential_Aws,
    GcsCredential_AwsId,
    GoogleChronicleCredential,
    GoogleChronicleCredential_OAuthClient,
    GoogleChronicleCredential_OAuthClientId,
    GoogleCredential,
    GoogleCredential_OAuthClient,
    GoogleCredential_OAuthClientId,
    GoogleServiceAccountCredential,
    GoogleServiceAccountCredential_OAuthClient,
    GoogleServiceAccountCredential_OAuthClientId,
    IdentityEntraId,
    IdentityGoogle,
    IdentityOkta,
    IdentityPingOne,
    JiraCredential,
    JiraCredential_Basic,
    JiraCredential_BasicId,
    MalwarebytesCredential,
    MalwarebytesCredential_OAuthClient,
    MalwarebytesCredential_OAuthClientId,
    NotificationsJira,
    NotificationsMock,
    NotificationsSlack,
    NotificationsSlackWebhook,
    NotificationsTeams,
    NozomiVantageCredential,
    NozomiVantageCredential_Basic,
    NozomiVantageCredential_BasicId,
    NucleusCredential,
    NucleusCredential_Token,
    NucleusCredential_TokenId,
    OktaCredential,
    OktaCredential_OAuthClient,
    OktaCredential_OAuthClientId,
    OktaCredential_Token,
    OktaCredential_TokenId,
    OpenSearchCredential,
    OpenSearchCredential_Basic,
    OpenSearchCredential_BasicId,
    PagerDutyCredential,
    PagerDutyCredential_Token,
    PagerDutyCredential_TokenId,
    PingOneCredential,
    PingOneCredential_Token,
    PingOneCredential_TokenId,
    ProviderConfig,
    ProviderConfigId,
    ProviderConfig_AssetsArmisCentrix,
    ProviderConfig_AssetsArmisCentrixMock,
    ProviderConfig_AssetsAxonius,
    ProviderConfig_AssetsCrowdstrike,
    ProviderConfig_AssetsNozomiVantage,
    ProviderConfig_AssetsNozomiVantageMock,
    ProviderConfig_AssetsQualysCloud,
    ProviderConfig_AssetsQualysCloudMock,
    ProviderConfig_AssetsServicenow,
    ProviderConfig_AssetsServicenowMock,
    ProviderConfig_AssetsSevco,
    ProviderConfig_AssetsTaniumCloud,
    ProviderConfig_AssetsTaniumCloudMock,
    ProviderConfig_CloudsecurityCrowdstrike,
    ProviderConfig_CloudsecurityDefender,
    ProviderConfig_EdrCrowdstrike,
    ProviderConfig_EdrDefender,
    ProviderConfig_EdrMalwarebytes,
    ProviderConfig_EdrSentinelone,
    ProviderConfig_EdrSophos,
    ProviderConfig_EdrTanium,
    ProviderConfig_IdentityEntraId,
    ProviderConfig_IdentityGoogle,
    ProviderConfig_IdentityOkta,
    ProviderConfig_IdentityPingone,
    ProviderConfig_NotificationsJira,
    ProviderConfig_NotificationsMockNotifications,
    ProviderConfig_NotificationsSlack,
    ProviderConfig_NotificationsSlackWebhook,
    ProviderConfig_NotificationsTeams,
    ProviderConfig_SiemCrowdstrike,
    ProviderConfig_SiemElasticsearch,
    ProviderConfig_SiemGoogleChronicle,
    ProviderConfig_SiemGoogleSecurityOperations,
    ProviderConfig_SiemMockSiem,
    ProviderConfig_SiemOpensearch,
    ProviderConfig_SiemQRadar,
    ProviderConfig_SiemRapid7Insightidr,
    ProviderConfig_SiemSentinel,
    ProviderConfig_SiemSplunk,
    ProviderConfig_SiemSumoLogic,
    ProviderConfig_SinkAwsSecurityLake,
    ProviderConfig_SinkAwsSqs,
    ProviderConfig_SinkAzureMonitorLogs,
    ProviderConfig_SinkCrowdstrikeHec,
    ProviderConfig_SinkElasticsearch,
    ProviderConfig_SinkGoogleSecOps,
    ProviderConfig_SinkGoogleSecurityOperations,
    ProviderConfig_SinkMockSink,
    ProviderConfig_SinkOpensearch,
    ProviderConfig_SinkSplunk,
    ProviderConfig_StorageAwsS3,
    ProviderConfig_StorageAzureBlob,
    ProviderConfig_StorageGcs,
    ProviderConfig_StorageMockStorage,
    ProviderConfig_TicketingAutotask,
    ProviderConfig_TicketingFreshdesk,
    ProviderConfig_TicketingJira,
    ProviderConfig_TicketingMockTicketing,
    ProviderConfig_TicketingPagerduty,
    ProviderConfig_TicketingServicenow,
    ProviderConfig_TicketingServicenowSir,
    ProviderConfig_TicketingTorq,
    ProviderConfig_TicketingZendesk,
    ProviderConfig_VulnerabilitiesCrowdstrike,
    ProviderConfig_VulnerabilitiesNucleus,
    ProviderConfig_VulnerabilitiesQualysCloud,
    ProviderConfig_VulnerabilitiesQualysCloudMock,
    ProviderConfig_VulnerabilitiesRapid7InsightCloud,
    ProviderConfig_VulnerabilitiesRapid7InsightCloudMock,
    ProviderConfig_VulnerabilitiesTaniumCloud,
    ProviderConfig_VulnerabilitiesTaniumCloudMock,
    ProviderConfig_VulnerabilitiesTenableCloud,
    QRadarCredential,
    QRadarCredential_Token,
    QRadarCredential_TokenId,
    QualysCloudCredential,
    QualysCloudCredential_Basic,
    QualysCloudCredential_BasicId,
    Rapid7InsightCloudCredential,
    Rapid7InsightCloudCredential_Token,
    Rapid7InsightCloudCredential_TokenId,
    SentinelCredential,
    SentinelCredential_OAuthClient,
    SentinelCredential_OAuthClientId,
    SentinelOneCredential,
    SentinelOneCredential_Token,
    SentinelOneCredential_TokenId,
    SentinelOneEdrEventsCredential,
    SentinelOneEdrEventsCredential_Token,
    SentinelOneEdrEventsCredential_TokenId,
    ServiceNowCredential,
    ServiceNowCredential_Basic,
    ServiceNowCredential_BasicId,
    ServiceNowCredential_Token,
    ServiceNowCredential_TokenId,
    SevcoCredential,
    SevcoCredential_Token,
    SevcoCredential_TokenId,
    SiemCrowdstrike,
    SiemElasticsearch,
    SiemGoogleChronicle,
    SiemGoogleSecurityOperations,
    SiemMock,
    SiemOpenSearch,
    SiemQRadar,
    SiemRapid7InsightIdr,
    SiemSentinel,
    SiemSplunk,
    SiemSumoLogic,
    SinkAwsSecurityLake,
    SinkAwsSqs,
    SinkAzureMonitorLogs,
    SinkCrowdstrikeHec,
    SinkElasticsearch,
    SinkGoogleSecOps,
    SinkGoogleSecurityOperations,
    SinkMock,
    SinkOpenSearch,
    SinkSplunk,
    SlackCredential,
    SlackCredential_Token,
    SlackCredential_TokenId,
    SlackWebhookCredential,
    SlackWebhookCredential_Secret,
    SlackWebhookCredential_SecretId,
    SophosCredential,
    SophosCredential_OAuthClient,
    SophosCredential_OAuthClientId,
    SplunkHecToken,
    SplunkHecToken_Token,
    SplunkHecToken_TokenId,
    SplunkSearchCredential,
    SplunkSearchCredential_Token,
    SplunkSearchCredential_TokenId,
    StorageAwsS3,
    StorageAzureBlob,
    StorageGcs,
    StorageMock,
    SumoLogicCollectionUrl,
    SumoLogicCollectionUrl_Secret,
    SumoLogicCollectionUrl_SecretId,
    SumoLogicCredential,
    SumoLogicCredential_Basic,
    SumoLogicCredential_BasicId,
    TaniumCloudCredential,
    TaniumCloudCredential_Token,
    TaniumCloudCredential_TokenId,
    TeamsCredential,
    TeamsCredential_OAuthClient,
    TeamsCredential_OAuthClientId,
    TeamsCredential_WebhookUrl,
    TeamsCredential_WebhookUrlId,
    TenableCloudCredential,
    TenableCloudCredential_Token,
    TenableCloudCredential_TokenId,
    TicketingAutotask,
    TicketingFreshdesk,
    TicketingJira,
    TicketingMock,
    TicketingPagerDuty,
    TicketingServiceNow,
    TicketingServiceNowSir,
    TicketingTorq,
    TicketingZendesk,
    TorqCredential,
    TorqCredential_OAuthClient,
    TorqCredential_OAuthClientId,
    VulnerabilitiesCrowdStrike,
    VulnerabilitiesNucleus,
    VulnerabilitiesQualysCloud,
    VulnerabilitiesQualysCloudDataset,
    VulnerabilitiesQualysCloudMock,
    VulnerabilitiesRapid7InsightCloud,
    VulnerabilitiesRapid7InsightCloudDataset,
    VulnerabilitiesRapid7InsightCloudMock,
    VulnerabilitiesTaniumCloud,
    VulnerabilitiesTaniumCloudDataset,
    VulnerabilitiesTaniumCloudMock,
    VulnerabilitiesTenableCloud,
    ZendeskCredential,
    ZendeskCredential_Basic,
    ZendeskCredential_BasicId,
)
from .role_base import AdhocRole, Resources, RoleAccounts, RoleId, RoleIntegrations, RoleName, RoleOrganizations
from .roles import (
    BuiltinRoles,
    CreateRoleRequest,
    CreateRoleResponse,
    GetRoleResponse,
    ListRolesResponse,
    PatchRoleResponse,
    RoleDefinition,
    UpdateRoleRequest,
    UpdateRoleResponse,
)
from .status import (
    GetIntegrationTimeseries,
    GetIntegrationTimeseriesResult,
    GetStatusResponse,
    GetStatusTimeseries,
    GetStatusTimeseriesResult,
    ListStatusEventsResponse,
    ListStatusOptions,
    ListStatusResponse,
    Status,
    StatusEvent,
    TimeseriesOptions,
    TimeseriesResult,
)
from .token_base import Token, TokenId, TokenOwnerType, TokenPair
from .tokens import (
    CreateIntegrationTokenRequest,
    CreateIntegrationTokenResponse,
    CreateSynqlyIntegrationsTokenRequest,
    CreateSynqlyIntegrationsTokenResponse,
    CreateTokenRequest,
    CreateTokenResponse,
    GetTokenResponse,
    ListTokensResponse,
    RefreshToken,
    RefreshTokenResponse,
    ResetTokenResponse,
    SynqlyIntegrationsTokenResponse,
)
from .usage import Usage

__all__ = [
    "Account",
    "AccountId",
    "AccountsActions",
    "AccountsPermissions",
    "ActionId",
    "AdditionalMapping",
    "AdditionalMappingTemplate",
    "AdditionalMappingType",
    "AdhocRole",
    "AlarmPoliciesActions",
    "AlarmPoliciesPermissions",
    "AlarmsActions",
    "AlarmsPermissions",
    "ApiPermissionMap",
    "ApplyMappingRequest",
    "ApplyMappingResponse",
    "ApplyMappingResponseResult",
    "ArmisCredential",
    "ArmisCredential_Token",
    "ArmisCredential_TokenId",
    "AssetsArmisCentrix",
    "AssetsArmisCentrixMock",
    "AssetsArmisDataset",
    "AssetsAxonius",
    "AssetsCrowdStrike",
    "AssetsNozomiVantage",
    "AssetsNozomiVantageDataset",
    "AssetsNozomiVantageMock",
    "AssetsQualysCloud",
    "AssetsQualysCloudDataset",
    "AssetsQualysCloudMock",
    "AssetsServiceNow",
    "AssetsServiceNowDataset",
    "AssetsServiceNowMock",
    "AssetsSevco",
    "AssetsTaniumCloud",
    "AssetsTaniumCloudDataset",
    "AssetsTaniumCloudMock",
    "AsyncOperationRequestId",
    "Audit",
    "AuditActions",
    "AuditPermissions",
    "AuditType",
    "AuthActions",
    "AuthCode",
    "AuthPermissions",
    "AutotaskApiIntegrationCodeCredential",
    "AutotaskApiIntegrationCodeCredential_Secret",
    "AutotaskApiIntegrationCodeCredential_SecretId",
    "AutotaskSecretCredential",
    "AutotaskSecretCredential_Secret",
    "AutotaskSecretCredential_SecretId",
    "AwsCredential",
    "AwsCredentialId",
    "AwsS3Credential",
    "AwsS3Credential_Aws",
    "AwsS3Credential_AwsId",
    "AwsSecurityLakeCredential",
    "AwsSecurityLakeCredential_Aws",
    "AwsSecurityLakeCredential_AwsId",
    "AwsSqsCredential",
    "AwsSqsCredential_Aws",
    "AwsSqsCredential_AwsId",
    "AxoniusCredential",
    "AxoniusCredential_Basic",
    "AxoniusCredential_BasicId",
    "AzureBlobCredential",
    "AzureBlobCredential_Token",
    "AzureBlobCredential_TokenId",
    "AzureMonitorLogsCredential",
    "AzureMonitorLogsCredential_Token",
    "AzureMonitorLogsCredential_TokenId",
    "BadGatewayError",
    "BadRequestError",
    "Base",
    "BasicCredential",
    "BasicCredentialId",
    "BillingActions",
    "BillingPermissions",
    "BridgeGroup",
    "BridgeGroupId",
    "BridgeLocalConfig",
    "BridgeLocalStats",
    "BridgeSelector",
    "BridgeSelector_Id",
    "BridgeSelector_Labels",
    "BridgeStatus",
    "BridgesActions",
    "BridgesPermissions",
    "BuiltinRoles",
    "CategoryId",
    "ChangePasswordRequest",
    "ChangePasswordResponse",
    "ChangePasswordResponseResult",
    "CloudSecurityCrowdStrike",
    "CloudSecurityDefender",
    "ConflictError",
    "Connector",
    "ConnectorOrId",
    "CreateAccountRequest",
    "CreateAccountResponse",
    "CreateAccountResponseResult",
    "CreateBridgeRequest",
    "CreateBridgeResponse",
    "CreateBridgeResponseResult",
    "CreateCredentialRequest",
    "CreateCredentialResponse",
    "CreateIntegrationPointRequest",
    "CreateIntegrationPointResponse",
    "CreateIntegrationRequest",
    "CreateIntegrationResponse",
    "CreateIntegrationResponseResult",
    "CreateIntegrationTokenRequest",
    "CreateIntegrationTokenResponse",
    "CreateMappingRequest",
    "CreateMappingResponse",
    "CreateMappingResponseResult",
    "CreateMemberRequest",
    "CreateMemberResponse",
    "CreateMemberResponseResult",
    "CreateOrganizationRequest",
    "CreateOrganizationResponse",
    "CreateOrganizationResponseResult",
    "CreateOrganizationWebhookRequest",
    "CreateOrganizationWebhookResponse",
    "CreateRoleRequest",
    "CreateRoleResponse",
    "CreateSynqlyIntegrationsTokenRequest",
    "CreateSynqlyIntegrationsTokenResponse",
    "CreateTokenRequest",
    "CreateTokenResponse",
    "Credential",
    "CredentialBase",
    "CredentialConfig",
    "CredentialConfigNoSecret",
    "CredentialConfig_Aws",
    "CredentialConfig_Basic",
    "CredentialConfig_OAuthClient",
    "CredentialConfig_Secret",
    "CredentialConfig_Token",
    "CredentialId",
    "CredentialResponse",
    "CredentialType",
    "CredentialsActions",
    "CredentialsPermissions",
    "CrowdStrikeCredential",
    "CrowdStrikeCredential_OAuthClient",
    "CrowdStrikeCredential_OAuthClientId",
    "CrowdstrikeHecCredential",
    "CrowdstrikeHecCredential_Token",
    "CrowdstrikeHecCredential_TokenId",
    "CustomFieldMapping",
    "DefenderCredential",
    "DefenderCredential_OAuthClient",
    "DefenderCredential_OAuthClientId",
    "EdrCrowdStrike",
    "EdrDefender",
    "EdrMalwarebytes",
    "EdrSentinelOne",
    "EdrSophos",
    "EdrTanium",
    "ElasticsearchAuthOptions",
    "ElasticsearchCredential",
    "ElasticsearchCredential_Basic",
    "ElasticsearchCredential_BasicId",
    "ElasticsearchCredential_OAuthClient",
    "ElasticsearchCredential_OAuthClientId",
    "ElasticsearchCredential_Token",
    "ElasticsearchCredential_TokenId",
    "ElasticsearchSharedSecret",
    "ElasticsearchSharedSecret_Secret",
    "ElasticsearchSharedSecret_SecretId",
    "EntraIdCredential",
    "EntraIdCredential_OAuthClient",
    "EntraIdCredential_OAuthClientId",
    "Environment",
    "FilterOperation",
    "FilterType",
    "ForbiddenError",
    "FreshdeskCredential",
    "FreshdeskCredential_Token",
    "FreshdeskCredential_TokenId",
    "GatewayTimeoutError",
    "GcsCredential",
    "GcsCredential_Aws",
    "GcsCredential_AwsId",
    "GetAccountResponse",
    "GetBridgeResponse",
    "GetBridgeStatusResponse",
    "GetCredentialResponse",
    "GetIntegrationPointResponse",
    "GetIntegrationResponse",
    "GetIntegrationTimeseries",
    "GetIntegrationTimeseriesResult",
    "GetMappingResponse",
    "GetMemberResponse",
    "GetOpenApiSpecResponse",
    "GetOrganizationResponse",
    "GetOrganizationWebhookResponse",
    "GetPermissionSetResponse",
    "GetProviderCapabilitiesExpandOptions",
    "GetRoleResponse",
    "GetStatusResponse",
    "GetStatusTimeseries",
    "GetStatusTimeseriesResult",
    "GetTokenResponse",
    "GoogleChronicleCredential",
    "GoogleChronicleCredential_OAuthClient",
    "GoogleChronicleCredential_OAuthClientId",
    "GoogleCredential",
    "GoogleCredential_OAuthClient",
    "GoogleCredential_OAuthClientId",
    "GoogleServiceAccountCredential",
    "GoogleServiceAccountCredential_OAuthClient",
    "GoogleServiceAccountCredential_OAuthClientId",
    "HttpMethod",
    "Id",
    "IdentityEntraId",
    "IdentityGoogle",
    "IdentityOkta",
    "IdentityPingOne",
    "Integration",
    "IntegrationEnvironments",
    "IntegrationId",
    "IntegrationPoint",
    "IntegrationPointId",
    "IntegrationPointsActions",
    "IntegrationPointsPermissions",
    "IntegrationsActions",
    "IntegrationsPermissions",
    "InternalServerError",
    "JiraCredential",
    "JiraCredential_Basic",
    "JiraCredential_BasicId",
    "ListAccountIntegrationsResponse",
    "ListAccountsResponse",
    "ListAuditEventsResponse",
    "ListBridgesResponse",
    "ListConnectorCapabilitiesExpandOptions",
    "ListConnectorsCapabilitiesResponse",
    "ListCredentialsResponse",
    "ListIntegrationOptions",
    "ListIntegrationPointsResponse",
    "ListIntegrationsResponse",
    "ListMappingsResponse",
    "ListMembersResponse",
    "ListOperationsResponse",
    "ListOrganizationResponse",
    "ListOrganizationWebhooksResponse",
    "ListPermissionSetsResponse",
    "ListProviderCapabilitiesExpandOptions",
    "ListProvidersCapabilitiesResponse",
    "ListRolesResponse",
    "ListStatusEventsResponse",
    "ListStatusOptions",
    "ListStatusResponse",
    "ListTokensResponse",
    "LogonRequest",
    "LogonResponse",
    "LogonResponseResult",
    "LookupCredentialResponse",
    "MalwarebytesCredential",
    "MalwarebytesCredential_OAuthClient",
    "MalwarebytesCredential_OAuthClientId",
    "ManagedType",
    "Mapping",
    "MappingBase",
    "MappingChain",
    "MappingChainTemplate",
    "MappingDataType",
    "MappingId",
    "MappingsActions",
    "MappingsPermissions",
    "Member",
    "MemberId",
    "MemberOptions",
    "MembersActions",
    "MembersPermissions",
    "MethodNotAllowedError",
    "NotFoundError",
    "NotImplementedError",
    "NotificationsJira",
    "NotificationsMock",
    "NotificationsSlack",
    "NotificationsSlackWebhook",
    "NotificationsTeams",
    "NozomiVantageCredential",
    "NozomiVantageCredential_Basic",
    "NozomiVantageCredential_BasicId",
    "NucleusCredential",
    "NucleusCredential_Token",
    "NucleusCredential_TokenId",
    "OAuthClientCredential",
    "OAuthClientCredentialId",
    "OktaCredential",
    "OktaCredential_OAuthClient",
    "OktaCredential_OAuthClientId",
    "OktaCredential_Token",
    "OktaCredential_TokenId",
    "OpenSearchCredential",
    "OpenSearchCredential_Basic",
    "OpenSearchCredential_BasicId",
    "Operation",
    "OperationError",
    "OperationId",
    "OperationInput",
    "OperationSchedule",
    "OperationStatus",
    "OperationsActions",
    "OperationsPermissions",
    "Options",
    "Organization",
    "OrganizationActions",
    "OrganizationId",
    "OrganizationOptions",
    "OrganizationPermissions",
    "OrganizationType",
    "OrganizationWebhook",
    "OrganizationWebhookPayload",
    "OrganizationWebhookSecret",
    "OwnerType",
    "PagerDutyCredential",
    "PagerDutyCredential_Token",
    "PagerDutyCredential_TokenId",
    "ParameterLocation",
    "PatchAccountResponse",
    "PatchBridgeResponse",
    "PatchCredentialResponse",
    "PatchIntegrationPointResponse",
    "PatchIntegrationResponse",
    "PatchMappingResponse",
    "PatchMemberResponse",
    "PatchOp",
    "PatchOperation",
    "PatchOrganizationResponse",
    "PatchOrganizationWebhookResponse",
    "PatchRoleResponse",
    "Permission",
    "PermissionSet",
    "PermissionSetActions",
    "PermissionSetPermissions",
    "Permissions",
    "PingOneCredential",
    "PingOneCredential_Token",
    "PingOneCredential_TokenId",
    "Problem",
    "ProblemCause",
    "ProblemContext",
    "ProblemDetails",
    "ProblemRelation",
    "ProblemType",
    "ProblematicParameter",
    "ProviderAvailability",
    "ProviderCapabilities",
    "ProviderCapabilitiesResponse",
    "ProviderConfig",
    "ProviderConfigId",
    "ProviderConfig_AssetsArmisCentrix",
    "ProviderConfig_AssetsArmisCentrixMock",
    "ProviderConfig_AssetsAxonius",
    "ProviderConfig_AssetsCrowdstrike",
    "ProviderConfig_AssetsNozomiVantage",
    "ProviderConfig_AssetsNozomiVantageMock",
    "ProviderConfig_AssetsQualysCloud",
    "ProviderConfig_AssetsQualysCloudMock",
    "ProviderConfig_AssetsServicenow",
    "ProviderConfig_AssetsServicenowMock",
    "ProviderConfig_AssetsSevco",
    "ProviderConfig_AssetsTaniumCloud",
    "ProviderConfig_AssetsTaniumCloudMock",
    "ProviderConfig_CloudsecurityCrowdstrike",
    "ProviderConfig_CloudsecurityDefender",
    "ProviderConfig_EdrCrowdstrike",
    "ProviderConfig_EdrDefender",
    "ProviderConfig_EdrMalwarebytes",
    "ProviderConfig_EdrSentinelone",
    "ProviderConfig_EdrSophos",
    "ProviderConfig_EdrTanium",
    "ProviderConfig_IdentityEntraId",
    "ProviderConfig_IdentityGoogle",
    "ProviderConfig_IdentityOkta",
    "ProviderConfig_IdentityPingone",
    "ProviderConfig_NotificationsJira",
    "ProviderConfig_NotificationsMockNotifications",
    "ProviderConfig_NotificationsSlack",
    "ProviderConfig_NotificationsSlackWebhook",
    "ProviderConfig_NotificationsTeams",
    "ProviderConfig_SiemCrowdstrike",
    "ProviderConfig_SiemElasticsearch",
    "ProviderConfig_SiemGoogleChronicle",
    "ProviderConfig_SiemGoogleSecurityOperations",
    "ProviderConfig_SiemMockSiem",
    "ProviderConfig_SiemOpensearch",
    "ProviderConfig_SiemQRadar",
    "ProviderConfig_SiemRapid7Insightidr",
    "ProviderConfig_SiemSentinel",
    "ProviderConfig_SiemSplunk",
    "ProviderConfig_SiemSumoLogic",
    "ProviderConfig_SinkAwsSecurityLake",
    "ProviderConfig_SinkAwsSqs",
    "ProviderConfig_SinkAzureMonitorLogs",
    "ProviderConfig_SinkCrowdstrikeHec",
    "ProviderConfig_SinkElasticsearch",
    "ProviderConfig_SinkGoogleSecOps",
    "ProviderConfig_SinkGoogleSecurityOperations",
    "ProviderConfig_SinkMockSink",
    "ProviderConfig_SinkOpensearch",
    "ProviderConfig_SinkSplunk",
    "ProviderConfig_StorageAwsS3",
    "ProviderConfig_StorageAzureBlob",
    "ProviderConfig_StorageGcs",
    "ProviderConfig_StorageMockStorage",
    "ProviderConfig_TicketingAutotask",
    "ProviderConfig_TicketingFreshdesk",
    "ProviderConfig_TicketingJira",
    "ProviderConfig_TicketingMockTicketing",
    "ProviderConfig_TicketingPagerduty",
    "ProviderConfig_TicketingServicenow",
    "ProviderConfig_TicketingServicenowSir",
    "ProviderConfig_TicketingTorq",
    "ProviderConfig_TicketingZendesk",
    "ProviderConfig_VulnerabilitiesCrowdstrike",
    "ProviderConfig_VulnerabilitiesNucleus",
    "ProviderConfig_VulnerabilitiesQualysCloud",
    "ProviderConfig_VulnerabilitiesQualysCloudMock",
    "ProviderConfig_VulnerabilitiesRapid7InsightCloud",
    "ProviderConfig_VulnerabilitiesRapid7InsightCloudMock",
    "ProviderConfig_VulnerabilitiesTaniumCloud",
    "ProviderConfig_VulnerabilitiesTaniumCloudMock",
    "ProviderConfig_VulnerabilitiesTenableCloud",
    "ProviderEnvironment",
    "ProviderFilter",
    "ProviderId",
    "ProviderOperations",
    "ProviderRelease",
    "QRadarCredential",
    "QRadarCredential_Token",
    "QRadarCredential_TokenId",
    "QualysCloudCredential",
    "QualysCloudCredential_Basic",
    "QualysCloudCredential_BasicId",
    "Rapid7InsightCloudCredential",
    "Rapid7InsightCloudCredential_Token",
    "Rapid7InsightCloudCredential_TokenId",
    "ReadWriteActions",
    "ReadWritePermissions",
    "RefreshToken",
    "RefreshTokenResponse",
    "RequestBody",
    "ResetTokenResponse",
    "ResourceId",
    "ResourceReference",
    "ResourceRestrictions",
    "ResourceType",
    "Resources",
    "RoleAccounts",
    "RoleDefinition",
    "RoleId",
    "RoleIntegrations",
    "RoleName",
    "RoleOrganizations",
    "RolesActions",
    "RolesPermissions",
    "SecretCredential",
    "SecretCredentialId",
    "SentinelCredential",
    "SentinelCredential_OAuthClient",
    "SentinelCredential_OAuthClientId",
    "SentinelOneCredential",
    "SentinelOneCredential_Token",
    "SentinelOneCredential_TokenId",
    "SentinelOneEdrEventsCredential",
    "SentinelOneEdrEventsCredential_Token",
    "SentinelOneEdrEventsCredential_TokenId",
    "ServiceNowCredential",
    "ServiceNowCredential_Basic",
    "ServiceNowCredential_BasicId",
    "ServiceNowCredential_Token",
    "ServiceNowCredential_TokenId",
    "ServiceUnavailableError",
    "SevcoCredential",
    "SevcoCredential_Token",
    "SevcoCredential_TokenId",
    "SiemCrowdstrike",
    "SiemElasticsearch",
    "SiemGoogleChronicle",
    "SiemGoogleSecurityOperations",
    "SiemMock",
    "SiemOpenSearch",
    "SiemQRadar",
    "SiemRapid7InsightIdr",
    "SiemSentinel",
    "SiemSplunk",
    "SiemSumoLogic",
    "SinkAwsSecurityLake",
    "SinkAwsSqs",
    "SinkAzureMonitorLogs",
    "SinkCrowdstrikeHec",
    "SinkElasticsearch",
    "SinkGoogleSecOps",
    "SinkGoogleSecurityOperations",
    "SinkMock",
    "SinkOpenSearch",
    "SinkSplunk",
    "SlackCredential",
    "SlackCredential_Token",
    "SlackCredential_TokenId",
    "SlackWebhookCredential",
    "SlackWebhookCredential_Secret",
    "SlackWebhookCredential_SecretId",
    "SophosCredential",
    "SophosCredential_OAuthClient",
    "SophosCredential_OAuthClientId",
    "SplunkHecToken",
    "SplunkHecToken_Token",
    "SplunkHecToken_TokenId",
    "SplunkSearchCredential",
    "SplunkSearchCredential_Token",
    "SplunkSearchCredential_TokenId",
    "State",
    "Status",
    "StatusActions",
    "StatusEvent",
    "StatusPermissions",
    "StorageAwsS3",
    "StorageAzureBlob",
    "StorageGcs",
    "StorageMock",
    "SubOrgsActions",
    "SubOrgsPermissions",
    "SumoLogicCollectionUrl",
    "SumoLogicCollectionUrl_Secret",
    "SumoLogicCollectionUrl_SecretId",
    "SumoLogicCredential",
    "SumoLogicCredential_Basic",
    "SumoLogicCredential_BasicId",
    "SynqlyIntegrationsTokenResponse",
    "TaniumCloudCredential",
    "TaniumCloudCredential_Token",
    "TaniumCloudCredential_TokenId",
    "TeamsCredential",
    "TeamsCredential_OAuthClient",
    "TeamsCredential_OAuthClientId",
    "TeamsCredential_WebhookUrl",
    "TeamsCredential_WebhookUrlId",
    "TenableCloudCredential",
    "TenableCloudCredential_Token",
    "TenableCloudCredential_TokenId",
    "TicketingAutotask",
    "TicketingFreshdesk",
    "TicketingJira",
    "TicketingMock",
    "TicketingPagerDuty",
    "TicketingServiceNow",
    "TicketingServiceNowSir",
    "TicketingTorq",
    "TicketingZendesk",
    "TimeseriesOptions",
    "TimeseriesResult",
    "Token",
    "TokenCredential",
    "TokenCredentialId",
    "TokenId",
    "TokenOwnerType",
    "TokenPair",
    "TokensActions",
    "TokensPermissions",
    "TooManyRequestsError",
    "TorqCredential",
    "TorqCredential_OAuthClient",
    "TorqCredential_OAuthClientId",
    "TransformsActions",
    "TransformsPermissions",
    "UnauthorizedError",
    "UnsupportedMediaTypeError",
    "UpdateAccountRequest",
    "UpdateAccountResponse",
    "UpdateBridgeRequest",
    "UpdateBridgeResponse",
    "UpdateCredentialRequest",
    "UpdateCredentialResponse",
    "UpdateIntegrationPointRequest",
    "UpdateIntegrationPointResponse",
    "UpdateIntegrationRequest",
    "UpdateIntegrationResponse",
    "UpdateMappingRequest",
    "UpdateMappingResponse",
    "UpdateMemberRequest",
    "UpdateMemberResponse",
    "UpdateOrganizationRequest",
    "UpdateOrganizationResponse",
    "UpdateOrganizationWebhookRequest",
    "UpdateOrganizationWebhookResponse",
    "UpdateRoleRequest",
    "UpdateRoleResponse",
    "Usage",
    "VerifyIntegrationRequest",
    "VulnerabilitiesCrowdStrike",
    "VulnerabilitiesNucleus",
    "VulnerabilitiesQualysCloud",
    "VulnerabilitiesQualysCloudDataset",
    "VulnerabilitiesQualysCloudMock",
    "VulnerabilitiesRapid7InsightCloud",
    "VulnerabilitiesRapid7InsightCloudDataset",
    "VulnerabilitiesRapid7InsightCloudMock",
    "VulnerabilitiesTaniumCloud",
    "VulnerabilitiesTaniumCloudDataset",
    "VulnerabilitiesTaniumCloudMock",
    "VulnerabilitiesTenableCloud",
    "WebhookConfig",
    "WebhookEvent",
    "WebhookFilter",
    "WebhookId",
    "WebhookItem",
    "WebhooksActions",
    "WebhooksPermissions",
    "ZendeskCredential",
    "ZendeskCredential_Basic",
    "ZendeskCredential_BasicId",
    "account_base",
    "accounts",
    "audit",
    "auth",
    "auth_base",
    "bridges",
    "capabilities",
    "capabilities_base",
    "common",
    "credentials",
    "integration_base",
    "integration_points",
    "integrations",
    "management",
    "mappings",
    "member_base",
    "members",
    "meta",
    "operation_base",
    "operation_id_generated",
    "operations",
    "organization",
    "organization_base",
    "organization_webhook_base",
    "organization_webhook_events",
    "organization_webhooks",
    "permissions",
    "permissionset",
    "permissionset_base",
    "providers_generated",
    "role_base",
    "roles",
    "status",
    "sub_orgs",
    "token_base",
    "tokens",
    "usage",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ParameterLocation(str, enum.Enum):
    HEADER = "header"
    PATH = "path"
    QUERY = "query"
    BODY = "body"

    def visit(
        self,
        header: typing.Callable[[], T_Result],
        path: typing.Callable[[], T_Result],
        query: typing.Callable[[], T_Result],
        body: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ParameterLocation.HEADER:
            return header()
        if self is ParameterLocation.PATH:
            return path()
        if self is ParameterLocation.QUERY:
            return query()
        if self is ParameterLocation.BODY:
            return body()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from .os_type_id import OsTypeId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Os(pydantic.BaseModel):
    """
    The Operating System (OS) object describes characteristics of an OS, such as Linux or Windows.
    """

    build: typing.Optional[str] = pydantic.Field(default=None)
    """
    The operating system build number.
    """

    country: typing.Optional[str] = pydantic.Field(default=None)
    """
    The operating system country code, as defined by the ISO 3166-1 standard (Alpha-2 code).<p><b>Note:</b> The two letter country code should be capitalized. For example: <code>US</code> or <code>CA</code>.</p>
    """

    cpe_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Common Platform Enumeration (CPE) name as described by (<a target='_blank' href='https://nvd.nist.gov/products/cpe'>NIST</a>) For example: <code>cpe:/a:apple:safari:16.2</code>.
    """

    cpu_bits: typing.Optional[int] = pydantic.Field(default=None)
    """
    The cpu architecture, the number of bits used for addressing in memory. For example: <code>32</code> or <code>64</code>.
    """

    edition: typing.Optional[str] = pydantic.Field(default=None)
    """
    The operating system edition. For example: <code>Professional</code>.
    """

    kernel_release: typing.Optional[str] = pydantic.Field(default=None)
    """
    The kernel release of the operating system. On Unix-based systems, this is determined from the <code>uname -r</code> command output, for example "5.15.0-122-generic".
    """

    lang: typing.Optional[str] = pydantic.Field(default=None)
    """
    The two letter lower case language codes, as defined by <a target='_blank' href='https://en.wikipedia.org/wiki/ISO_639-1'>ISO 639-1</a>. For example: <code>en</code> (English), <code>de</code> (German), or <code>fr</code> (French).
    """

    name: str = pydantic.Field()
    """
    The operating system name.
    """

    sp_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the latest Service Pack.
    """

    sp_ver: typing.Optional[int] = pydantic.Field(default=None)
    """
    The version number of the latest Service Pack.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of the operating system.
    """

    type_id: OsTypeId = pydantic.Field()
    """
    The type identifier of the operating system.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The version of the OS running on the device that originated the event. For example: "Windows 10", "OS X 10.7", or "iOS 9".
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.email_address import EmailAddress
from .auth_factor_factor_type_id import AuthFactorFactorTypeId
from .device import Device

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class AuthFactor(pydantic.BaseModel):
    """
    An Authentication Factor object describes a category of methods used for identity verification in an authentication attempt.
    """

    device: typing.Optional[Device] = pydantic.Field(default=None)
    """
    Device used to complete an authentication request.
    """

    email_addr: typing.Optional[EmailAddress] = pydantic.Field(default=None)
    """
    The email address used in an email-based authentication factor.
    """

    factor_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of authentication factor used in an authentication attempt.
    """

    factor_type_id: AuthFactorFactorTypeId = pydantic.Field()
    """
    The normalized identifier for the authentication factor.
    """

    is_hotp: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the authentication factor is an HMAC-based One-time Password (HOTP).
    """

    is_totp: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the authentication factor is a Time-based One-time Password (TOTP).
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The phone number used for a telephony-based authentication request.
    """

    provider: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of provider for an authentication factor.
    """

    security_questions: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The question(s) provided to user for a question-based authentication factor.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class LongString(pydantic.BaseModel):
    """
    This object is a used to capture strings which may be truncated by a security product due to their length.
    """

    is_truncated: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates that <code>value</code> has been truncated. May be omitted if truncation has not occurred.
    """

    untruncated_size: typing.Optional[int] = pydantic.Field(default=None)
    """
    The size in bytes of the string represented by <code>value</code> before truncation. Should be omitted if truncation has not occurred.
    """

    value: str = pydantic.Field()
    """
    The string value, truncated if <code>is_truncated</code> is <code>true</code>.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class IntegrationTicketWhen(str, enum.Enum):
    AFTER = "after"
    ASYNC = "async"
    BEFORE = "before"
    DISPLAY = "display"

    def visit(
        self,
        after: typing.Callable[[], T_Result],
        async_: typing.Callable[[], T_Result],
        before: typing.Callable[[], T_Result],
        display: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is IntegrationTicketWhen.AFTER:
            return after()
        if self is IntegrationTicketWhen.ASYNC:
            return async_()
        if self is IntegrationTicketWhen.BEFORE:
            return before()
        if self is IntegrationTicketWhen.DISPLAY:
            return display()



# This file was auto-generated by Fern from our API Definition.

from .bridge_group import BridgeGroup
from .bridge_group_id import BridgeGroupId
from .bridge_local_config import BridgeLocalConfig
from .bridge_local_stats import BridgeLocalStats
from .bridge_status import BridgeStatus
from .create_bridge_request import CreateBridgeRequest
from .create_bridge_response import CreateBridgeResponse
from .create_bridge_response_result import CreateBridgeResponseResult
from .get_bridge_response import GetBridgeResponse
from .get_bridge_status_response import GetBridgeStatusResponse
from .list_bridges_response import ListBridgesResponse
from .patch_bridge_response import PatchBridgeResponse
from .update_bridge_request import UpdateBridgeRequest
from .update_bridge_response import UpdateBridgeResponse

__all__ = [
    "BridgeGroup",
    "BridgeGroupId",
    "BridgeLocalConfig",
    "BridgeLocalStats",
    "BridgeStatus",
    "CreateBridgeRequest",
    "CreateBridgeResponse",
    "CreateBridgeResponseResult",
    "GetBridgeResponse",
    "GetBridgeStatusResponse",
    "ListBridgesResponse",
    "PatchBridgeResponse",
    "UpdateBridgeRequest",
    "UpdateBridgeResponse",
]



# This file was auto-generated by Fern from our API Definition.

from ...ocsf.resources.v_1_3_0.resources.detectionfinding.resources.classes.types.detection_finding import (
    DetectionFinding,
)

ThreatEvent = DetectionFinding



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...ocsf.resources.v_1_3_0.resources.securityfinding.resources.classes.types.severity_id import SeverityId
from .provider_specific_finding_state import ProviderSpecificFindingState
from .vulnerability_severity_filter_value import VulnerabilitySeverityFilterValue
from .vulnerability_state_filter_value import VulnerabilityStateFilterValue

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class UpdateFindingRequest(pydantic.BaseModel):
    severity_id: SeverityId
    severity: typing.Optional[VulnerabilitySeverityFilterValue] = pydantic.Field(default=None)
    """
    severity of the finding
    """

    state: typing.Optional[VulnerabilityStateFilterValue] = pydantic.Field(default=None)
    """
    state of the finding
    """

    unmapped: typing.Optional[ProviderSpecificFindingState] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class VulnerabilitySeverityFilterValue(str, enum.Enum):
    """
    Values supported by using severity as a filter. Supports `[eq]` and `[in]` operators.
    For example, `severity[eq]critical` or `severity[in]critical, high`.
    """

    CRITICAL = "critical"
    """
    Action is required immediately and the scope is broad.
    """

    HIGH = "high"
    """
    Action is required immediately.
    """

    MEDIUM = "medium"
    """
    Action is required but the situation is not serious at this time.
    """

    LOW = "low"
    """
    The user decides if action is needed.
    """

    INFO = "info"
    """
    Informational message. No action required.
    """

    def visit(
        self,
        critical: typing.Callable[[], T_Result],
        high: typing.Callable[[], T_Result],
        medium: typing.Callable[[], T_Result],
        low: typing.Callable[[], T_Result],
        info: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is VulnerabilitySeverityFilterValue.CRITICAL:
            return critical()
        if self is VulnerabilitySeverityFilterValue.HIGH:
            return high()
        if self is VulnerabilitySeverityFilterValue.MEDIUM:
            return medium()
        if self is VulnerabilitySeverityFilterValue.LOW:
            return low()
        if self is VulnerabilitySeverityFilterValue.INFO:
            return info()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ProcessorArchitectureOv(str, enum.Enum):
    ALPHA = "alpha"
    ARM = "arm"
    AI_64 = "ai_64"
    MIPS = "mips"
    POWERPC = "powerpc"
    SPARC = "sparc"
    X_86 = "x86"
    X_86_64 = "x86_64"

    def visit(
        self,
        alpha: typing.Callable[[], T_Result],
        arm: typing.Callable[[], T_Result],
        ai_64: typing.Callable[[], T_Result],
        mips: typing.Callable[[], T_Result],
        powerpc: typing.Callable[[], T_Result],
        sparc: typing.Callable[[], T_Result],
        x_86: typing.Callable[[], T_Result],
        x_86_64: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ProcessorArchitectureOv.ALPHA:
            return alpha()
        if self is ProcessorArchitectureOv.ARM:
            return arm()
        if self is ProcessorArchitectureOv.AI_64:
            return ai_64()
        if self is ProcessorArchitectureOv.MIPS:
            return mips()
        if self is ProcessorArchitectureOv.POWERPC:
            return powerpc()
        if self is ProcessorArchitectureOv.SPARC:
            return sparc()
        if self is ProcessorArchitectureOv.X_86:
            return x_86()
        if self is ProcessorArchitectureOv.X_86_64:
            return x_86_64()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class TokenOwnerType(str, enum.Enum):
    ORGANIZATION = "organization"
    INTEGRATION = "integration"

    def visit(
        self, organization: typing.Callable[[], T_Result], integration: typing.Callable[[], T_Result]
    ) -> T_Result:
        if self is TokenOwnerType.ORGANIZATION:
            return organization()
        if self is TokenOwnerType.INTEGRATION:
            return integration()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ReadWriteActions(str, enum.Enum):
    READ = "read"
    READ_WRITE = "*"

    def visit(self, read: typing.Callable[[], T_Result], read_write: typing.Callable[[], T_Result]) -> T_Result:
        if self is ReadWriteActions.READ:
            return read()
        if self is ReadWriteActions.READ_WRITE:
            return read_write()



# This file was auto-generated by Fern from our API Definition.

from .types import PatchOrganizationResponse, UpdateOrganizationRequest, UpdateOrganizationResponse

__all__ = ["PatchOrganizationResponse", "UpdateOrganizationRequest", "UpdateOrganizationResponse"]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...capabilities_base.types.category_id import CategoryId
from ...capabilities_base.types.provider_id import ProviderId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Connector(pydantic.BaseModel):
    """
    Provides details of the Connector.
    """

    id: CategoryId = pydantic.Field()
    """
    Unique identifier for the Connector.
    """

    name: CategoryId = pydantic.Field()
    """
    Name of the connector.
    """

    fullname: str = pydantic.Field()
    """
    Display name of the Connector.
    """

    description: str = pydantic.Field()
    """
    Description of the Connector.
    """

    provider_ids: typing.List[ProviderId] = pydantic.Field()
    """
    List of Providers that implement the Connector.
    """

    providers: typing.Optional[typing.List[ProviderCapabilities]] = pydantic.Field(default=None)
    """
    List of capabilities for Providers that implement the Connector.
    """

    connector: typing.Optional[CategoryId] = pydantic.Field(default=None)
    """
    DEPRECATED – use `id` instead.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}


from .provider_capabilities import ProviderCapabilities  # noqa: E402

Connector.update_forward_refs()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.process_name import ProcessName
from ...base.types.timestamp import Timestamp

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ProcessEntity(pydantic.BaseModel):
    """
    The Process Entity object provides critical fields for referencing a process.
    """

    cmd_line: typing.Optional[str] = pydantic.Field(default=None)
    """
    The full command line used to launch an application, service, process, or job. For example: <code>ssh user@10.0.0.10</code>. If the command line is unavailable or missing, the empty string <code>''</code> is to be used.
    """

    created_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the process was created/started.
    """

    created_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the process was created/started.
    """

    name: typing.Optional[ProcessName] = pydantic.Field(default=None)
    """
    The friendly name of the process, for example: <code>Notepad++</code>.
    """

    path: typing.Optional[str] = pydantic.Field(default=None)
    """
    The process file path.
    """

    pid: typing.Optional[int] = pydantic.Field(default=None)
    """
    The process identifier, as reported by the operating system. Process ID (PID) is a number used by the operating system to uniquely identify an active process.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique identifier for this process assigned by the producer (tool). Facilitates correlation of a process event with other events for that process.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_dict import remove_none_from_dict
from ...core.request_options import RequestOptions
from ..common.errors.bad_gateway_error import BadGatewayError
from ..common.errors.bad_request_error import BadRequestError
from ..common.errors.conflict_error import ConflictError
from ..common.errors.forbidden_error import ForbiddenError
from ..common.errors.gateway_timeout_error import GatewayTimeoutError
from ..common.errors.internal_server_error import InternalServerError
from ..common.errors.method_not_allowed_error import MethodNotAllowedError
from ..common.errors.not_found_error import NotFoundError
from ..common.errors.not_implemented_error import NotImplementedError
from ..common.errors.service_unavailable_error import ServiceUnavailableError
from ..common.errors.too_many_requests_error import TooManyRequestsError
from ..common.errors.unauthorized_error import UnauthorizedError
from ..common.errors.unsupported_media_type_error import UnsupportedMediaTypeError
from ..common.types.problem import Problem
from .types.asset import Asset
from .types.create_asset_request import CreateAssetRequest
from .types.create_asset_response import CreateAssetResponse
from .types.create_findings_request import CreateFindingsRequest
from .types.create_findings_response import CreateFindingsResponse
from .types.get_scan_activity_response import GetScanActivityResponse
from .types.query_assets_response import QueryAssetsResponse
from .types.query_findings_response import QueryFindingsResponse
from .types.query_scans_response import QueryScansResponse
from .types.update_finding_request import UpdateFindingRequest

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class VulnerabilitiesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def query_findings(
        self,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryFindingsResponse:
        """
        Query vulnerability findings

        Parameters:
            - meta: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

            - limit: typing.Optional[int]. Number of finding reports to return. Defaults to 50.

            - cursor: typing.Optional[str]. Start search from cursor position.

            - filter: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Filter results by this query. For more information on filtering, refer to the Vulnerability Filtering Guide.
                                                                                Defaults to no filter. If used more than once, the queries are ANDed together.
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/vulnerabilities/findings"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "meta": meta,
                        "limit": limit,
                        "cursor": cursor,
                        "filter": filter,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(QueryFindingsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_findings(
        self, *, request: CreateFindingsRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateFindingsResponse:
        """
        Create findings (bulk) in a vulnerability scanning system

        Parameters:
            - request: CreateFindingsRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/vulnerabilities/findings/bulk"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateFindingsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_finding(
        self, finding_id: str, *, request: UpdateFindingRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        update a finding in a vulnerability scanning system

        Parameters:
            - finding_id: str. Uid of the Finding (URL encoded). This will be `finding.uid` in the OCSF model.

            - request: UpdateFindingRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/vulnerabilities/findings/{jsonable_encoder(finding_id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def query_assets(
        self,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryAssetsResponse:
        """
        Query assets in a vulnerability scanning system

        Parameters:
            - meta: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

            - limit: typing.Optional[int]. Number of assets to return. Defaults to 50.

            - cursor: typing.Optional[str]. Start search from cursor position.

            - filter: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Filter results by this query. For more information on filtering, refer to the Vulnerability Filtering Guide.
                                                                                Defaults to no filter. If used more than once, the queries are ANDed together.
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/vulnerabilities/assets"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "meta": meta,
                        "limit": limit,
                        "cursor": cursor,
                        "filter": filter,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(QueryAssetsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_asset(
        self, *, request: CreateAssetRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[CreateAssetResponse]:
        """
        Create assets in a vulnerability scanning system

        Parameters:
            - request: CreateAssetRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/vulnerabilities/assets"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.Optional[CreateAssetResponse], _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_asset(
        self, asset_id: str, *, request: CreateAssetRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> Asset:
        """
        update an asset in a vulnerability scanning system

        Parameters:
            - asset_id: str. Uid of the Asset. This will be `devices.uid` in the OCSF model.

            - request: CreateAssetRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/vulnerabilities/assets/{jsonable_encoder(asset_id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Asset, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def query_scans(
        self,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryScansResponse:
        """
        Query scans in a vulnerability scanning system

        Parameters:
            - meta: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

            - limit: typing.Optional[int]. Number of scans to return. Defaults to 50.

            - cursor: typing.Optional[str]. Start search from cursor position.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/vulnerabilities/scans"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "meta": meta,
                        "limit": limit,
                        "cursor": cursor,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(QueryScansResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_scan_activity(
        self, scan_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetScanActivityResponse:
        """
        Get a list of activity generated by a configured scan.

        Parameters:
            - scan_id: str. ID of the scan to get its activity.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"v1/vulnerabilities/scans/{jsonable_encoder(scan_id)}/activity",
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetScanActivityResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncVulnerabilitiesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def query_findings(
        self,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryFindingsResponse:
        """
        Query vulnerability findings

        Parameters:
            - meta: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

            - limit: typing.Optional[int]. Number of finding reports to return. Defaults to 50.

            - cursor: typing.Optional[str]. Start search from cursor position.

            - filter: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Filter results by this query. For more information on filtering, refer to the Vulnerability Filtering Guide.
                                                                                Defaults to no filter. If used more than once, the queries are ANDed together.
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/vulnerabilities/findings"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "meta": meta,
                        "limit": limit,
                        "cursor": cursor,
                        "filter": filter,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(QueryFindingsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_findings(
        self, *, request: CreateFindingsRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateFindingsResponse:
        """
        Create findings (bulk) in a vulnerability scanning system

        Parameters:
            - request: CreateFindingsRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/vulnerabilities/findings/bulk"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CreateFindingsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_finding(
        self, finding_id: str, *, request: UpdateFindingRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        update a finding in a vulnerability scanning system

        Parameters:
            - finding_id: str. Uid of the Finding (URL encoded). This will be `finding.uid` in the OCSF model.

            - request: UpdateFindingRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/vulnerabilities/findings/{jsonable_encoder(finding_id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def query_assets(
        self,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryAssetsResponse:
        """
        Query assets in a vulnerability scanning system

        Parameters:
            - meta: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

            - limit: typing.Optional[int]. Number of assets to return. Defaults to 50.

            - cursor: typing.Optional[str]. Start search from cursor position.

            - filter: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Filter results by this query. For more information on filtering, refer to the Vulnerability Filtering Guide.
                                                                                Defaults to no filter. If used more than once, the queries are ANDed together.
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/vulnerabilities/assets"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "meta": meta,
                        "limit": limit,
                        "cursor": cursor,
                        "filter": filter,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(QueryAssetsResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_asset(
        self, *, request: CreateAssetRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Optional[CreateAssetResponse]:
        """
        Create assets in a vulnerability scanning system

        Parameters:
            - request: CreateAssetRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/vulnerabilities/assets"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.Optional[CreateAssetResponse], _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_asset(
        self, asset_id: str, *, request: CreateAssetRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> Asset:
        """
        update an asset in a vulnerability scanning system

        Parameters:
            - asset_id: str. Uid of the Asset. This will be `devices.uid` in the OCSF model.

            - request: CreateAssetRequest.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/vulnerabilities/assets/{jsonable_encoder(asset_id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Asset, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def query_scans(
        self,
        *,
        meta: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryScansResponse:
        """
        Query scans in a vulnerability scanning system

        Parameters:
            - meta: typing.Optional[typing.Union[str, typing.Sequence[str]]]. Add metadata to the response by invoking meta functions. Documentation for meta functions is available at https://docs.synqly.com/api-reference/meta-functions. Not all meta function are available at every endpoint.

            - limit: typing.Optional[int]. Number of scans to return. Defaults to 50.

            - cursor: typing.Optional[str]. Start search from cursor position.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/vulnerabilities/scans"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "meta": meta,
                        "limit": limit,
                        "cursor": cursor,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(QueryScansResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_scan_activity(
        self, scan_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetScanActivityResponse:
        """
        Get a list of activity generated by a configured scan.

        Parameters:
            - scan_id: str. ID of the scan to get its activity.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"v1/vulnerabilities/scans/{jsonable_encoder(scan_id)}/activity",
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GetScanActivityResponse, _response.json())  # type: ignore
        if _response.status_code == 400:
            raise BadRequestError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 403:
            raise ForbiddenError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 404:
            raise NotFoundError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 405:
            raise MethodNotAllowedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 409:
            raise ConflictError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 415:
            raise UnsupportedMediaTypeError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 429:
            raise TooManyRequestsError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 500:
            raise InternalServerError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 501:
            raise NotImplementedError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 502:
            raise BadGatewayError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 503:
            raise ServiceUnavailableError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        if _response.status_code == 504:
            raise GatewayTimeoutError(pydantic.parse_obj_as(Problem, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..........core.datetime_utils import serialize_datetime
from ...base.types.timestamp import Timestamp
from ...base.types.url_string import UrlString
from .advisory_install_state_id import AdvisoryInstallStateId
from .cve import Cve
from .cwe import Cwe
from .os import Os
from .product import Product
from .timespan import Timespan

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Advisory(pydantic.BaseModel):
    """
    The Advisory object represents publicly disclosed cybersecurity vulnerabilities defined in a Secruity advisory. e.g. <code> Microsoft KB Article, Apple Security Advisory</code>
    """

    avg_timespan: typing.Optional[Timespan] = pydantic.Field(default=None)
    """
    The average time to patch.
    """

    bulletin: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Advisory bulletin identifier.
    """

    classification: typing.Optional[str] = pydantic.Field(default=None)
    """
    The vendors classification of the Advisory.
    """

    created_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the Advisory record was created.
    """

    created_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the Advisory record was created.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    A brief description of the Advisory Record.
    """

    install_state: typing.Optional[str] = pydantic.Field(default=None)
    """
    The install state of the Advisory.
    """

    install_state_id: typing.Optional[AdvisoryInstallStateId] = pydantic.Field(default=None)
    """
    The normalized install state ID of the Advisory.
    """

    is_superseded: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The Advisory has been replaced by another.
    """

    modified_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the Advisory record was last updated.
    """

    modified_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the Advisory record was last updated.
    """

    os: typing.Optional[Os] = pydantic.Field(default=None)
    """
    The operating system the Advisory applies to.
    """

    product: typing.Optional[Product] = pydantic.Field(default=None)
    """
    The product where the vulnerability was discovered.
    """

    references: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of reference URLs with additional information about the vulnerabilities disclosed in the Advisory.
    """

    related_cves: typing.Optional[typing.List[Cve]] = pydantic.Field(default=None)
    """
    A list of Common Vulnerabilities and Exposures <a target='_blank' href='https://cve.mitre.org/'>(CVE)</a> identifiers related to the vulnerabilities disclosed in the Advisory.
    """

    related_cwes: typing.Optional[typing.List[Cwe]] = pydantic.Field(default=None)
    """
    A list of Common Weakness Enumeration <a target='_blank' href='https://cwe.mitre.org/'>(CWE)</a> identifiers related to the vulnerabilities disclosed in the Advisory.
    """

    size: typing.Optional[int] = pydantic.Field(default=None)
    """
    The size in bytes for the Advisory. Usually populated for a KB Article patch.
    """

    src_url: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    The Advisory link from the source vendor.
    """

    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    A title or a brief phrase summarizing the Advisory.
    """

    uid: str = pydantic.Field()
    """
    The unique number assigned to the disclosed vulnerability.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}
